# -*- coding: iso_8859_1 -*-

from astropython import astro
import numpy as np
import matplotlib.pyplot as plt
import os, time

#-------------------------------------------------------------------------------------------
#	this scripts detects all new FITS files written to the specified folder
#	it computes their
#	- min, mean and max values
#	- median PSF fwhm (small, large) over the detected stars
#	- X and Y shifts wrt a specified reference image
#-------------------------------------------------------------------------------------------

#path + file information
path_to_watch = u"C:\\images\\"
reference_image = "image-1.fit"

#fit parameters
detection_area = 12
nb_stars = 12

#-------------------------------------------------------------------------------------------

before = dict ([(f, None) for f in os.listdir (path_to_watch)]) 

#first file to be used as a reference for drift
image_ref = astro.get_imagedata([path_to_watch+reference_image])
astro.display(image_ref)
print
print 'Define asteroid position so that it is removed from shift calculation'
x0 = np.int(raw_input('x position (vertical): x0 = '))
y0 = np.int(raw_input('y position (horizontal): y0 = '))
plt.close()

#loop to detect new fits files created in folder
while 1: 
	time.sleep (10) 
	after = dict ([(f, None) for f in os.listdir (path_to_watch)]) 
	added = [f for f in after if not f in before] 
	#if added: print "Added: ", ", ".join (added) 	
	before = after 

	for f in added:
		if astro.check_extension(f):
			#loading new images
			image = astro.get_imagedata([path_to_watch+f])
			
			#calculating statistics
			mean = np.mean(image.flatten())
			maxi = np.max(image.flatten())
			mini = np.min(image.flatten())
			print  "                   min: ", mini, "     max: ", maxi, "     mean: ", mean

			#fitting stars
			th = astro.find_threshold(image, detection_area, nb_stars)
			fwhm_small, fwhm_large, angle = astro.mean_psf(image, th, detection_area)
			print "                   FWHM small = %.2f" % fwhm_small
			print "                   FWHM large = %.2f" % fwhm_large

			#finding shift wrt reference image
			dx, dy = astro.find_shift_stars_matching(image_ref, image, th, detection_area, excluded=[[x0-10, x0+10, y0-10, y0+10]])
			print "                   X and Y shifts = ", dx, dy
			print 
