# -*- coding: iso_8859_1 -*-

from astropython import astro, photometry

#------------------------------------------------------------------------------------
#	input parameters (basic)
#------------------------------------------------------------------------------------

#--- calibrated images ---
Folder = u"C:\\images\\"
Imagelist = [Folder + "images-calibrated-" + str(i+1) + ".fit" for i in range(600)]

#radii for aperture photometry (pixels)
sizes = [[1.5,8,40],[2,8,40],[2.5,8,40],[3,8,40],[3.5,8,40],[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40]]
#sizes = [[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40],[6.5,10,45],[7,10,45],[7.5,12,45],[8,12,45],[8.5,14,50],[9,14,50]]
#sizes = [[8,16,45],[8.5,16,50],[9,16,50],[9.5,16,55],[10,18,50],[10.5,18,50],[11,18,50],[11.5,18,50],[12,20,50]]

#--- names of output files ---
generic_name = Folder + 'photometry_test-'

#--- semi-size of square area used for star detection, star fitting, etc ---
detection_area = 8			#typically a few times (2-3) the star FWHM

#------------------------------------------------------------------------------------
#	input parameters (advanced)
#------------------------------------------------------------------------------------

#--- photometry method ---
phot = 'recenter'
#phot = 'normal'

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_selected_stars.pdf"
outputname4 = generic_name + "04_inst_magnitudes.txt"
outputname5 = generic_name + "05_best_diff_magnitudes.txt"
outputname6 = generic_name + "06_CDRCDLreport.txt"

#####################################################################################
#	SCRIPT STARTS HERE
#####################################################################################
#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

#------ checking quality of measured frames ------

print
print '---------------------------------------------------------------------'
print '      Step 1 : checking the input images'
print '---------------------------------------------------------------------'
print
Imagelist2 = photometry.check_pictures_quality(Imagelist, outputname1)

#------ calculating shifts ------

print
print '---------------------------------------------------------------------'
print '      Step 2 : calculation of image shifts'
print '---------------------------------------------------------------------'
print
Imagelist3, shifts = photometry.calculate_shifts(Imagelist2, outputname2, detection_area = detection_area, nb_stars = 12)

#------------------------------------------------------------------------------------
#	calculates instrumental magnitudes
#------------------------------------------------------------------------------------

#------ date information ------
dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))

#------ select stars and perform aperture photometry ------
if phot=='normal':
	print
	print '---------------------------------------------------------------------'
	print '      Step 3 : selecting targets for photometry'
	print '---------------------------------------------------------------------'
	print
	names, xy, xy2 = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = detection_area, psf_fit = False)
	print
	print '---------------------------------------------------------------------'
	print '      Step 4 : calculating aperture photometry'
	print '---------------------------------------------------------------------'
	print
	mag = photometry.aperture_photometry(Imagelist3, sizes, xy, method=phot)
	photometry.write_magnitudes(Imagelist3, outputname4, names, xy, dateobs, sizes, mag)
else:
	print
	print '---------------------------------------------------------------------'
	print '      Step 3 : selecting targets for photometry'
	print '---------------------------------------------------------------------'
	print
	names, xy, xy2 = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = detection_area, psf_fit = True)
	print
	print '---------------------------------------------------------------------'
	print '      Step 4 : calculating aperture photometry'
	print '---------------------------------------------------------------------'
	print
	mag = photometry.aperture_photometry(Imagelist3, sizes, xy2, method=phot)
	photometry.write_magnitudes(Imagelist3, outputname4, names, xy2, dateobs, sizes, mag)	

#------------------------------------------------------------------------------------
#	checking reference stars
#------------------------------------------------------------------------------------

print
print '---------------------------------------------------------------------'
print '      Step 5 : checking reference stars'
print '---------------------------------------------------------------------'
print

nb_ref = len([i for i in names if (i[0]=='r')])

if nb_ref==1:
	print
	print "Only one reference stars selected.\nScript is stopped without calculating differential magnitudes..."
	ref_checked = False
else:			
	ref_no = photometry.check_refstars(Imagelist3, names, mag)

	if (nb_ref-len(ref_no)<=1):	#the ref star check cannot be consistent
		print
		print "Not enough remaining reference stars.\nScript is stopped without calculating differential magnitudes..."
		ref_checked = False
	else:
		ref_checked = True

#------------------------------------------------------------------------------------
#	calculating differential magnitudes
#------------------------------------------------------------------------------------

nb_mov = len([i for i in names if (i[0]=='m')])
nb_vbl = len([i for i in names if (i[0]=='v')])
	

if ref_checked:
	nb_best = 10
	
	print
	print '---------------------------------------------------------------------'
	print '      Step 6 : calculating optimal differential magnitudes'
	print '---------------------------------------------------------------------'
	print

	for i in range(nb_mov+nb_vbl):
	
		rej_sizes, rej_data = photometry.filter_magnitudes(Imagelist3, names, i, sizes, mag, ref_no)		
		ref_1, mag_1, sig_1 = photometry.best_magnitude(Imagelist3, names, i, sizes, mag, ref_no, nb=nb_best, rejdata = rej_data, rejsizes = rej_sizes)
		photometry.write_best_magnitude(outputname5, Imagelist3, names, i, sizes, ref_1, mag_1, sig_1, dateobs, nb=nb_best, rejdata = rej_data, rejsizes = rej_sizes)
		photometry.write_cdrcdl_report(outputname6, Imagelist3, names, i, mag, ref_1, dateobs, rejdata = rej_data)

raw_input("ok")
