# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *

#----------------------------------------------------------------------------------------
#def add_sigma_clipping(images, sigma, iterations):
#
#--- purpose ---
#to combine a list of images with the sigma clipping method
#the method works pixel by pixel: for one pixel, it calculates the mean and std_deviation values on all images, then the pixels deviating from the mean by sigma_coef * std_deviation are rejected
#the process is repeated iterations times, then the mean pixel value is kept for the final image
#the method is not recommended for a low number (<15) of images. For a higher number of images it is very efficient
#----------------------------------------------------------------------------------------

def add_sigma_clipping2(images, sigma, iterations):

	dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim_x*dim_y) 
	z=imag_out()

	sigma_dll = c_double(sigma)

	cdll.sigma_clipping.Sigma_clipping(dim,dim_x,dim_y,y,sigma_dll,iterations,byref(z))

	result = []
	result[:] = z[:]

	return np.reshape(result,(dim_x,dim_y))

def add_sigma_clipping(images, sigma, iterations, cutsize=[500,500]):
	t0 = time.time()
	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = dim_x/sx + 1
	ny = dim_y/sy + 1
	
	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_sigma_clipping2(images[:,i1:i2,j1:j2], sigma, iterations)
	print time.time()-t0
	return result

#----------------------------------------------------------------------------------------
#def add_mean(images):
#
#--- purpose ---
#computes the mean image from a list of images
#----------------------------------------------------------------------------------------

def add_mean(images):
	return np.mean(images, axis=0)
	
#----------------------------------------------------------------------------------------
#def add_median(images):
#
#--- purpose ---
#computes the median image from a list of images
#the algorithm to calculate the median values is based on the Wirth method
#a C implementation from Devillard (from http://ndevilla.free.fr/median/median/index.html) is used, after correction of a typo
#the original code is compiled as a DLL, the DLL is called from Python with the module c_types
#----------------------------------------------------------------------------------------

def add_median2(images):

	dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------

	imag_out = c_double * (dim_x*dim_y) 
	z=imag_out()

	cdll.median.Median(dim,dim_x,dim_y,y,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def add_median(images, cutsize=[1000,1000]):

	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = dim_x/sx + 1
	ny = dim_y/sy + 1
	
	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_median2(images[:,i1:i2,j1:j2])

	return result

