# -*- coding: iso_8859_1 -*-

#import matplotlib
#matplotlib.use('GTKAgg') 

import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib.widgets import Slider

import numpy as np
import time, warnings
from ctypes import *
import win32con
import stats

#----------------------------------------------------------------------------------------
#def display(data, title=''):
#
#--- purpose ---
#to display one FITS image
#----------------------------------------------------------------------------------------

def display(data, title=''):

	data = np.array(data,dtype=np.int)
	med = stats.get_median(data)[0]
	dsig = np.std(data.flatten())
	
	plt.ion()
	fig = plt.figure()
	ax = plt.subplot(111)
	plt.subplots_adjust(bottom=0.2)

	h = plt.imshow(data, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
	
	#------ defining sliders ------
	axmini = plt.axes([0.1, 0.05, 0.8, 0.03])
	axmaxi  = plt.axes([0.1, 0.1, 0.8, 0.03])

	smini = Slider(axmini, 'min', med-5*dsig, med+20*dsig, dragging=False, valinit=med-0.5*dsig, valfmt='%1.0f')
	smaxi = Slider(axmaxi, 'max', med-5*dsig, med+20*dsig, dragging=False, valinit=med+2*dsig, valfmt='%1.0f')

	smini.poly.set_color('0.5')
	smaxi.poly.set_color('0.5')
	def update(val):
		mini = smini.val
		maxi = smaxi.val
		if mini<=maxi:
			dat = data
			h.set_data(dat)
			h.set_clim(vmin=smini.val, vmax=smaxi.val)
		else:
			dat = maxi+mini-data
			h.set_data(dat)
			h.set_clim(vmin=smaxi.val, vmax=smini.val)
			
		plt.draw()

	smini.on_changed(update)
	smaxi.on_changed(update)

	if title!='':
		fig.canvas.set_window_title(title) 
	plt.draw()
	return fig, ax
	
#----------------------------------------------------------------------------------------
#def select(fig, ax, nb, color='none'): 
#
#--- purpose ---
#to select an arbitrary number of (x,y) positions on a displayed image
#usually this function is used to select stars
#selections are made with a mouse right-click, so that the left click can still be used to zoom in and out in the image
#----------------------------------------------------------------------------------------

def select(fig, ax, nb, color='none'): 

        xy = [] 
        cid = 0 
        warnings.simplefilter("ignore") 
        global i 
        i=0 

        #------------ 
        def on_click(event): 
                global i 
                if event.inaxes is not None: 
                        if event.button==3: 
                                xy.append([int(round(event.ydata)),int(round(event.xdata))]) 
                                i = i+1 
                                if color != 'none': 
										ax.scatter(xy[-1][1],xy[-1][0],s=50, marker='o', edgecolors=color, facecolor='none') 
										ax.annotate(str(i), xy=(xy[-1][1]+10, xy[-1][0]+10),  xycoords='data', color=color) 
										plt.draw() 
                                print 'x=',xy[-1][0],'y=',xy[-1][1] 
                                if len(xy)==nb: 
                                        fig.canvas.mpl_disconnect(cid) 
                                        fig.canvas.stop_event_loop_default() 
                else: 
                        if nb<0: 
                                if event.button==3: 
                                        fig.canvas.mpl_disconnect(cid) 
                                        fig.canvas.stop_event_loop_default() 

        cid = fig.canvas.mpl_connect('button_press_event', on_click) 
        fig.canvas.start_event_loop_default() 

        warnings.resetwarnings() 
        time.sleep(0.25) 
        return xy
