# -*- coding: iso_8859_1 -*-

import datetime, time
import numpy as np
import load
import astropython.conf as conf

#----------------------------------------------------------------------------------------
#def create_headers(Config, Imagelist):
#
#--- purpose ---
#to create a list of FITS headers from scratch, that can be passed to the function astro.save_imagelist() to save a new FITS image
#----------------------------------------------------------------------------------------

def create_headers(Config, Imagelist):

	hdictconfig = conf.set_headers(Config)
	hdictlist = load.get_headers(Imagelist, text_display=False)		#list of header dictionaries from image list
	hdictlistnew = []												#new list of header dictionaries

	for hdict in hdictlist:
		hdictnew = hdictconfig
		for key in hdictnew.keys():
			if hdictnew[key]==[]:
				if key in hdict:
					hdictnew[key] = hdict[key]
				else:
					del hdictnew[key]

		hdictlistnew.append(hdictnew)

	return hdictlistnew

#----------------------------------------------------------------------------------------
#def julian_day(dateobs):
#
#--- purpose ---
#to convert FITS header DATE-OBS, provided in format "2011-03-21T23:12:34.323", to a Julian day
#digit after the "." separator, representing a fraction of a second, can be present or not. If they are present, they will be considered
#----------------------------------------------------------------------------------------

def julian_day(dateobs):
	time_format = "%Y-%m-%dT%H:%M:%S"
	result = []
	for dd in dateobs:
		dd2 = dd.split('.')[0]
		dd3 = datetime.datetime.fromtimestamp(time.mktime(time.strptime(dd2, time_format)))

		year = dd3.year
		month = dd3.month
		
		day = dd3.day + dd3.hour/24. + dd3.minute/(24.*60.) + dd3.second/(24.*3600.)
		
		if len(dd.split('.'))>1:
			if len(dd.split('.')[1])>0:
				day = day + float('.'+dd.split('.')[1])/(24.*3600.)

		if month<=2:
			month = month+12
			year = year-1
		
		a = np.trunc(year/100.)
		b = 2. - a + np.trunc(a/4.)
			
		result.append(np.trunc(365.25*year)+np.trunc(30.6001*(month+1.))+day+1720994.5+b)
	return result