# -*- coding: iso_8859_1 -*-

import pyfits
import numpy as np
import os
import astropython.conf as conf

#----------------------------------------------------------------------------------------
#def get_imagedata(Imagelist, text_display=True):
#
#--- purpose ---
#to load the image data of a serie of FITS files
#----------------------------------------------------------------------------------------

def get_imagedata(Imagelist, text_display=True):

	if text_display:
		print
		print "--- loading FITS image ---"
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
			
		try:
			bzero = hdulist[0].header['bzero']
		except KeyError:
			bzero = 0.
		conf.update_saturation(32767. + bzero)

		if i==0:
			imagedata = np.array(hdulist[0].data, dtype=np.float32)
			if len(Imagelist)==1:
				imagedata = np.tile(imagedata,(1,1))
			else:
				imagedata = np.tile(imagedata,(len(Imagelist),1,1))
		else:
			imagedata[i,:,:] = np.array(hdulist[0].data, dtype=np.float32)

	hdulist.close()
	return imagedata

def get_someimagedata(Imagelist, area, text_display=True):

	if text_display:
		print
		print "--- loading part of FITS image ---"
		print "loaded pixels:[",area[0],":",area[1],",",area[2],":",area[3],"]"
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
			
		try:
			bzero = hdulist[0].header['bzero']
		except KeyError:
			bzero = 0.
		conf.update_saturation(32767. + bzero)
		
		if i==0:
			imagedata = hdulist[0].section[area[0]:area[1],area[2]:area[3]]
			if len(Imagelist)==1:
				imagedata = np.tile(imagedata,(1,1))
			else:
				imagedata = np.tile(imagedata,(len(Imagelist),1,1))
		else:
			imagedata[i,:,:] = hdulist[0].section[area[0]:area[1],area[2]:area[3]]

	hdulist.close()
		
	return np.array(imagedata, dtype=np.float32)
	
#----------------------------------------------------------------------------------------
#def get_headers(Imagelist, text_display=True):
#
#--- purpose ---
#to load all FITS headers of a serie of FITS images
#----------------------------------------------------------------------------------------

def get_headers(Imagelist, text_display=True):
	
	if text_display:
		print 
		print "--- loading FITS headers ---"
	listheaders = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
		
		cardlist = hdulist[0].header.ascardlist()
		headerdict = {}
		for x in cardlist:
			headerdict[x.key] = [x.value, x.comment]

		if i==0:
			listheaders = [headerdict]
		else:
			listheaders.append(headerdict)

	hdulist.close()
	return listheaders

#----------------------------------------------------------------------------------------
#def get_headervalues(Imagelist, Header):
#
#--- purpose ---
#to load all the values of a specific FITS header for a serie of FITS images
#----------------------------------------------------------------------------------------

def get_headervalues(Imagelist, Header):
	
	headervalues = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		try:
			hdulist = pyfits.open(sImage)
			headervalues.append(hdulist[0].header[Header])
		except KeyError:
			headervalues.append('')

	hdulist.close()
		
	return headervalues

#----------------------------------------------------------------------------------------
#def check_extension(file_name):
#
#--- purpose ---
#to check if a given file has the ".fit" or ".fits" extension
#----------------------------------------------------------------------------------------

def check_extension(file_name):
	ext = os.path.splitext(file_name)[1]
	if (ext==".fit") or (ext==".fits"):
		answer = True
	else:
		answer = False
	return answer