# -*- coding: iso_8859_1 -*-

import numpy as np
import process, process2
from ctypes import *

#----------------------------------------------------------------------------------------
#def get_median(images):
#
#--- purpose ---
#calculates the median values of all pixels on each image in a list of images
#if the image data are integer, they are converted to float before computation in order to get the half integer values correctly
#the algorithm to calculate the median values is based on the Wirth method
#a C implementation from Devillard (from http://ndevilla.free.fr/median/median/index.html) is used, after correction of a typo
#the original code is compiled as a DLL, the DLL is called from Python with the module c_types
#----------------------------------------------------------------------------------------

def get_median(images):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------

	imag_out = c_double * (dim) 
	z=imag_out()

	cdll.median.Median2(dim,dim_x,dim_y,y,byref(z))

	result = []
	result[:] = z[:]
	return result
	
#----------------------------------------------------------------------------------------
#def mean_psf(image, threshold, detection_area, psf_fit_area):
#
#--- purpose ---
#calculates the median parameters of the PSF of the brightest stars on one image
#the function is using the 2D elliptical Gaussian PSF model (similar to the Gaussian PSF used in PRISM software)
#----------------------------------------------------------------------------------------

def mean_psf(image, threshold, detection_area):

	#fitting stars
	stars = process2.find_stars_and_more(image, threshold, detection_area, full_info=True)
	
	#picking information
	fwhm_1 = []
	fwhm_2 = []
	cosine = []
	sine = []

	for xy in stars:
		#print xy
		fwhm_1.append(xy[5])
		fwhm_2.append(xy[6])
		cosine.append(np.cos(xy[7]))
		sine.append(np.sin(xy[7]))
		
	return [np.ma.mean(fwhm_1), np.ma.mean(fwhm_2), np.arctan2(np.ma.mean(sine),np.ma.mean(cosine))]
