# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *

#----------------------------------------------------------------------------------------
#def translate_bilinear(images, shift):
#def rotate_bilinear(images, xy_axis, angle):
#def scale_bilinear(images, xy_center, scaling):
#def translate_Bspline(images, shift, spline_degree):
#def rotate_Bspline(images, xy_axis, angle, spline_degree):
#def scale_Bspline(images, xy_center, scaling, spline_degree):
#
#--- purpose ---
#these functions apply a non-integer translation, a rotation or scaling to one or a list of input images
#B-spline interpolation is re-using existing C code from http://bigwww.epfl.ch/thevenaz/interpolation/
#----------------------------------------------------------------------------------------

def translate_Bspline(images, shift, spline_degree):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape
		
	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.Bspline.Translate(dim,dim_x,dim_y,y,byref(dxy),spline_degree,byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def rotate_Bspline(images, xy_axis, angle, spline_degree):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.Bspline.Rotate(dim,dim_x,dim_y,y,byref(xy_dll),angle_dll,spline_degree,byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def scale_Bspline(images, xy_center, scaling, spline_degree):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]

	scaling_dll = c_double(scaling)
	
	cdll.Bspline.Scale(dim,dim_x,dim_y,y,byref(xy_dll),scaling_dll,spline_degree,byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))		
	return result

def translate_bilinear(images, shift):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.bilinear.Translate(dim,dim_x,dim_y,y,byref(dxy),byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def rotate_bilinear(images, xy_axis, angle):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.bilinear.Rotate(dim,dim_x,dim_y,y,byref(xy_dll),angle_dll,byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def scale_bilinear(images, xy_center, scaling):

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	x = images.flatten()
	c_double_p = POINTER(c_double)
	x = x.astype(np.double)
	y = x.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]
	
	scaling_dll = c_double(scaling)
	
	cdll.bilinear.Scale(dim,dim_x,dim_y,y,byref(xy_dll),scaling_dll,byref(z))

	result = []
	result[:] = z[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

