# -*- coding: iso_8859_1 -*-

#------------------------------------------------------------------------------------------------------------
#		Calls to Win32 API functions to manage windows positions, size, etc
#------------------------------------------------------------------------------------------------------------

from ctypes import *
import win32con

#----------------------------------------------------------------------------------------
#def set_focus(title):
#def maximize_window(title):
#def scroll_up_window(title):
#
#--- purpose ---
#acts on the window with specified title: set focus, maximize, scroll up
#
#--- input parameters ---
#title = string giving window title
#		
#--- output parameters ---
#none
#----------------------------------------------------------------------------------------
def set_focus(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SetForegroundWindow(h)
	return 0
	
def maximize_window(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.ShowWindow(h, win32con.SW_MAXIMIZE)
	return 0

def scroll_up_window(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SetScrollPos(h, win32con.SB_VERT, 0, 1)
	windll.User32.SendMessageA(h, win32con.WM_VSCROLL, win32con.SB_TOP, 0)
	return 0

def close_window(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	windll.User32.SendMessageA(h, win32con.WM_CLOSE, 0, 0)
	return 0
	
#----------------------------------------------------------------------------------------
#def get_window_size(title):
#def get_window_position(title):
#
#--- purpose ---
#gets information about window with specified title: size or position
#
#--- input parameters ---
#title = string giving window title
#		
#--- output parameters ---
#two elements list [x,y], giving the window size or position
#----------------------------------------------------------------------------------------

def get_window_size(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.user32.GetWindowRect(h, byref(s))
	return [s[2]-s[0], s[3]-s[1]]

def get_window_position(title):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.user32.GetWindowRect(h, byref(s))
	return [s[0], s[1]]		#position of top left corner

#----------------------------------------------------------------------------------------
#def move_window(title, xy):
#def resize_window(title, xy):
#
#--- purpose ---
#moves or resizes the window with specified title
#
#--- input parameters ---
#title = string giving window title
#xy = 2 elements list [x,y] giving the new window size or position	
#
#--- output parameters ---
#none
#----------------------------------------------------------------------------------------
	
def move_window(title, xy):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.User32.GetWindowRect(h, byref(s))
	windll.User32.MoveWindow(h, xy[0], xy[1], s[2]-s[0], s[3]-s[1], 1)
	return 0

def resize_window(title, xy):
	h = windll.User32.FindWindowA(None, c_char_p(title))
	rect = c_long * 4
	s = rect()
	windll.User32.GetWindowRect(h, byref(s))
	windll.User32.MoveWindow(h, s[0], s[1], xy[0], xy[1], 1)
	return 0

#----------------------------------------------------------------------------------------
#def get_screen_size():
#
#--- purpose ---
#gets the screen size in pixels
#
#--- input parameters ---
#none
#
#--- output parameters ---
#two elements list [x,y] giving the screen size
#----------------------------------------------------------------------------------------

def get_screen_size():
	x = windll.User32.GetSystemMetrics(win32con.SM_CXSCREEN)
	y = windll.User32.GetSystemMetrics(win32con.SM_CYSCREEN)
	return [x,y]

