# -*- coding: iso_8859_1 -*-

from astropython import astro, conf
import numpy as np
import calib7

def calibrate_coalign_coadd_and_save(Imagelist, Darklist, Flat, mult, coadded_Image, Config, detection_area=10, nb_stars=12, excl=[], incl=[], text=False, method='mean', Headers={}):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = calib7.find_hotpixels(dark_final, 500)
	
	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat / astro.get_median(flat)
	flat = flat + 1e-10

	#------ calibration of 1st image ------
	image0 = astro.get_imagedata([Imagelist[0]])
	image0_calib = (image0 - dark_final) / flat
	image0_avg = (np.roll(image0_calib,1,axis=0) + np.roll(image0_calib,-1,axis=0) + np.roll(image0_calib,1,axis=1) + np.roll(image0_calib,-1,axis=1)) / 4.
	image0_calib = image0_avg*hotpixels + image0_calib*(1.-hotpixels)

	#------ checking for saturation ------
	test = (image0_calib<conf.saturation-0.001)*(image0!=conf.saturation)
	image0_calib = (1.-test)*conf.saturation + test*image0_calib
	image0_calib = (image0_calib>=0.)*image0_calib							#this step is required for coalignment with star matching (for star detection) otherwise mean is too low due to values << 0
	print 'dark and flat calibration ... done'
	print 'shifts of image 1 wrt image 1 (vert, hor):', 0., 0.	

	images_aligned = np.zeros([len(Imagelist)]+list(np.shape(image0)))
	images_aligned[0,:,:] = image0_calib

	th0 = astro.find_threshold(image0_calib, detection_area, nb_stars, excluded=excl, included=incl)
	for i in range(1,len(Imagelist)):

		#loading and calibration
		imagei = astro.get_imagedata([Imagelist[i]])	
		imagei_calib = (imagei - dark_final) / flat
		imagei_avg = (np.roll(imagei_calib,1,axis=0) + np.roll(imagei_calib,-1,axis=0) + np.roll(imagei_calib,1,axis=1) + np.roll(imagei_calib,-1,axis=1)) / 4.
		imagei_calib = imagei_avg*hotpixels + imagei_calib*(1.-hotpixels)

		#checking for saturation
		test = (imagei_calib<conf.saturation-0.001)*(imagei!=conf.saturation)
		imagei_calib = (1.-test)*conf.saturation + test*imagei_calib
		imagei_calib = (imagei_calib>=0.)*imagei_calib						#this step is required for coalignment with star matching (for star detection) otherwise mean is too low due to values << 0
		print 'dark and flat calibration ... done'

		#calculate shifts
		thi = astro.find_threshold(imagei_calib, detection_area, nb_stars, excluded=excl, included=incl)
		threshold = min(th0, thi)		
		dx,dy = astro.find_shift_stars_matching(image0_calib, imagei_calib, threshold, detection_area, excluded = excl, included = incl, text_display = text)
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(imagei_calib, (-dx,-dy), 3)

	if method == 'mean':
		image_final = mult*astro.add_mean(images_aligned)
	elif method == 'median':
		image_final = mult*astro.add_median(images_aligned)
	elif method == 'sigma clipping':
		image_final = mult*astro.add_sigma_clipping(images_aligned,3.,1)

	#save image
	if astro.get_headervalues([Imagelist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Imagelist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Imagelist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(image_final, [headerdict], [coadded_Image])
	print
	print coadded_Image + ' saved'
		
	return image_final

def calibrate_coalign2_coadd_and_save(Imagelist, Darklist, Flat, mult, coadded_Image, Config, method='mean', Headers={}):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = calib7.find_hotpixels(dark_final, 500)
	
	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat / astro.get_median(flat)
	flat = flat + 1e-10

	#------ calibration of 1st image ------
	image0 = astro.get_imagedata([Imagelist[0]])
	image0_calib = (image0 - dark_final) / flat
	image0_avg = (np.roll(image0_calib,1,axis=0) + np.roll(image0_calib,-1,axis=0) + np.roll(image0_calib,1,axis=1) + np.roll(image0_calib,-1,axis=1)) / 4.
	image0_calib = image0_avg*hotpixels + image0_calib*(1.-hotpixels)

	#------ checking for saturation ------
	test = (image0_calib<conf.saturation-0.001)*(image0!=conf.saturation)
	image0_calib = (1.-test)*conf.saturation + test*image0_calib
	print 'dark and flat calibration ... done'
	print 'shifts of image 1 wrt image 1 (vert, hor):', 0., 0.	

	images_aligned = np.zeros([len(Imagelist)]+list(np.shape(image0)))
	images_aligned[0,:,:] = image0_calib

	for i in range(1,len(Imagelist)):

		#loading and calibration
		imagei = astro.get_imagedata([Imagelist[i]])	
		imagei_calib = (imagei - dark_final) / flat
		imagei_avg = (np.roll(imagei_calib,1,axis=0) + np.roll(imagei_calib,-1,axis=0) + np.roll(imagei_calib,1,axis=1) + np.roll(imagei_calib,-1,axis=1)) / 4.
		imagei_calib = imagei_avg*hotpixels + imagei_calib*(1.-hotpixels)

		#checking for saturation
		test = (imagei_calib<conf.saturation-0.001)*(imagei!=conf.saturation)
		imagei_calib = (1.-test)*conf.saturation + test*imagei_calib
		print 'dark and flat calibration ... done'

		#calculate shifts
		dx, dy = astro.find_shift_crosscorrelation(image0_calib, imagei_calib)
		print
		print 'shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(imagei_calib, (-dx,-dy), 3)

	if method == 'mean':
		image_final = mult*astro.add_mean(images_aligned)
	elif method == 'median':
		image_final = mult*astro.add_median(images_aligned)
	elif method == 'sigma clipping':
		image_final = mult*astro.add_sigma_clipping(images_aligned,3.,1)

	#save image
	if astro.get_headervalues([Imagelist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Imagelist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Imagelist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Imagelist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(image_final, [headerdict], [coadded_Image])
	print
	print coadded_Image + ' saved'
		
	return image_final
