# -*- coding: iso_8859_1 -*-

from astropython import astro, conf
import numpy as np
import calib7

def process_flatframe_constantexp(Flatlist, Darklist, Finalflat, Config, Headers={}):

	#------ generating dark ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	del darks
	hotpixels = calib7.find_hotpixels(dark_final, 500)	
	print
	print "dark final generated ... ok"

	#------ generating flat ------
	flats = astro.get_imagedata(Flatlist)
	for i in range(len(Flatlist)):
		flats[i,:,:] = flats[i,:,:] - dark_final	
		flats[i,:,:] = flats[i,:,:] * 20000. / astro.get_median(flats[i,:,:])
	flat_final = astro.add_median(flats)
	del flats
	flat_avg = (np.roll(flat_final,1,axis=0) + np.roll(flat_final,-1,axis=0) + np.roll(flat_final,1,axis=1) + np.roll(flat_final,-1,axis=1)) / 4.
	flat_final = flat_avg*hotpixels + flat_final*(1.-hotpixels)	
	print "flat final generated ... ok"
	print "max value (ADU)=", np.max(flat_final.flatten())

	#astro.display(flat_final, title='Final flat frame')

	#------ saving final image.fit ------
	if astro.get_headervalues([Flatlist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Flatlist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Flatlist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Flatlist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(flat_final, [headerdict], [Finalflat])
	print "flat final saved to", Finalflat

	return flat_final

def process_flatframe_vbleexp(Flatlist, Darklist, Offsetlist, Finalflat, Config, Headers={}):

	#------ generating offset and dark ------
	offsets = astro.get_imagedata(Offsetlist)
	offset_final = astro.add_median(offsets)
	del offsets
	print
	print "offset final generated ... ok"
	darks = astro.get_imagedata(Darklist)
	thermic_final = astro.add_median(darks) - offset_final
	del darks
	hotpixels = calib7.find_hotpixels(thermic_final, 500)	
	print "thermic final generated ... ok"

	#------ generating flat ------
	flats = astro.get_imagedata(Flatlist)
	Exp_dark = astro.get_headervalues(Darklist, 'EXPOSURE')[0]
	if Exp_dark=='':
		Exp_dark = astro.get_headervalues(Darklist, 'EXPTIME')[0]
	Exp_flat = astro.get_headervalues(Flatlist, 'EXPOSURE')
	if Exp_flat[0]=='':
		Exp_flat = astro.get_headervalues(Flatlist, 'EXPTIME')
	for i in range(len(Flatlist)):
		flats[i,:,:] = flats[i,:,:] - offset_final - thermic_final * Exp_flat[i]/Exp_dark
		flats[i,:,:] = flats[i,:,:] * 20000. / astro.get_median(flats[i,:,:])
	flat_final = astro.add_median(flats)
	del flats
	flat_avg = (np.roll(flat_final,1,axis=0) + np.roll(flat_final,-1,axis=0) + np.roll(flat_final,1,axis=1) + np.roll(flat_final,-1,axis=1)) / 4.
	flat_final = flat_avg*hotpixels + flat_final*(1.-hotpixels)			
	print "flat final generated ... ok"
	print "max value (ADU)=", np.max(flat_final.flatten())

	#astro.display(flat_final, title='Final flat frame')

	#------ saving final image.fit ------
	if astro.get_headervalues([Flatlist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Flatlist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Flatlist, 'DATE-END'))[-1]
	headerdict = astro.create_headers(Config, [Flatlist[0]])[0]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update(Headers)
	headerdict.update({'EXPOSURE':[Exp,'Exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(flat_final, [headerdict], [Finalflat])
	print "flat final saved to", Finalflat

	return flat_final
