# -*- coding: iso_8859_1 -*-

import cdr1
import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate

def load_magdata(Filelist, light_correction = [False,'filename'], plotdata = [False,'Title']):

	print
	print "------ loading measured data ------" 

	alldata = []

	for ff in Filelist:
		data, offset = cdr1.load_magfile(ff)
		
		#checking reference stars
		#plt.figure()
		#for i in range(np.shape(data)[1]-2):
		#	refstar = data[:,i+2]-np.mean(data[:,2:],axis=1) 
		#	plt.plot(refstar-np.mean(refstar)+0.02*np.double(i),label=str(i+1))
		#plt.legend()
		#plt.grid(True)
		#plt.show()
		
		#shifting date to mid-exposure
		data[:,0] = data[:,0] + offset/86400.

		#calculating differential magnitude
		data[:,1] = data[:,1] - np.mean(data[:,2:],axis=1)
		data = data[:,:2]
		#setting average magnitude to 0
		data[:,1] = data[:,1] - np.mean(data[:,1])
		
		alldata.append(data)

	if plotdata[0]:
		plt.figure()
		for dd in alldata:
			plt.plot(dd[:,0],dd[:,1]-np.mean(dd[:,1]))
		plt.title(plotdata[1])
		plt.grid()
		plt.show()

		#checking reference stars
		#plt.figure()
		#for i in range(np.shape(data)[1]-2):
		#	refstar = data[:,i+2]-np.mean(data[:,2:],axis=1) 
		#	plt.plot(refstar-np.mean(refstar)+0.02*np.double(i),label=str(i+1))
		#plt.legend()
		#plt.grid(True)
		#plt.show()

	if light_correction[0]:

		print
		print "------ correcting for light time propagation ------" 

		sec_per_au = 149598000. / 299792.458
		dist = cdr1.load_file(light_correction[1])
		cubic_interp = interpolate.interp1d(dist[:,0], dist[:,1], kind='cubic')

		for i,dd in enumerate(alldata):
			dist_Earth = cubic_interp(dd[:,0])
			alldata[i][:,0] = dd[:,0] - dist_Earth * sec_per_au / 86400.

	print
	print "------ evaluating noise in measurements ------" 

	#measurement data
	xdata = np.zeros((0.))
	ydata = np.zeros((0.))
	sigmadata = np.zeros((0.))
	indices = np.array([0])

	for i,dd in enumerate(alldata):
		noise = cdr1.calc_noise(dd[:,1])
		noise = np.tile(dd[:,0]*0.+noise,(1,1))
		alldata[i] = np.concatenate((dd.transpose(),noise)).transpose()

	return alldata

