# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize


def fit_CDR(alldata, period, T0, nb_harmo = 6, show_results = True, show_plot = True):

	print
	print "------ least square fit with Fourier series ------" 

	#fit parameters
	offsets = np.zeros((len(alldata)))
	an = np.ones((nb_harmo))
	bn = np.ones((nb_harmo))
	p = [period] + list(an) + list(bn) + list(offsets)
		
	#fit function
	def CDR(alldata, *p):
		#global T0
		result = np.array([])
		for i,dd in enumerate(alldata):
			arg = (dd[:,0]-T0)*2.*np.pi/p[0]
			fit = p[i+1+2*nb_harmo]
			for j in range(nb_harmo):
				fit = fit + p[j+1]*np.cos(np.double(j+1)*arg) + p[j+1+nb_harmo]*np.sin(np.double(j+1)*arg)
			result = np.hstack((result,np.array(fit)))
		return result
		
	#re-organizing measurement data
	ydata = np.zeros((0.))
	sigmadata = np.zeros((0.))

	for i,dd in enumerate(alldata):
		ydata = np.hstack((ydata,dd[:,1]))
		sigmadata = np.hstack((sigmadata,dd[:,2]))

	#fitting
	popt, pcov = optimize.curve_fit(CDR, alldata, ydata, p0=p, sigma = sigmadata)

	if show_results:
		print
		print "------ fit results ------"	
		print popt
		print pcov

	if show_plot:
		plt.figure()
		for i,dd in enumerate(alldata):
			plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0]), dd[:,1]-popt[i+1+2*nb_harmo],'+')
		x = np.linspace(0.,1.,1000)
		y = 0.
		for j in range(nb_harmo):
			y = y + popt[j+1]*np.cos(np.double(j+1)*x*2.*np.pi) + popt[j+1+nb_harmo]*np.sin(np.double(j+1)*x*2.*np.pi)
		plt.plot(x,y,'b')
		plt.xlim([0.,1.])
		plt.grid()
		plt.xlabel('Phase')
		plt.ylabel('Magnitude')

	return popt, pcov