/*****************************************************************************
 *	Includes / define
 ****************************************************************************/
//system includes
#include	<math.h>
#include	<stdio.h>
#include	<stdlib.h>

//other includes
#include	"dll.h"

//constants
#define		PI	((double)3.14159265358979323846264338327950288419716939937510)

/*****************************************************************************
 *	Definition of main functions
 ****************************************************************************/

extern "C"
{
__declspec(dllexport) int Translate(int dim, int dim_x, int dim_y, double* a, double* shifts, int spline_degree, double* b)
{
    float *buffer = new float[dim_x*dim_y];

	double x0, y0;
	int Error;

    for (int p=0; p<dim; p++)
    {
        //puts image p in array buffer
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            buffer[i*dim_y + j] = a[p*dim_x*dim_y + i*dim_y + j];
        }}

        /* convert between a representation based on image samples */
        /* and a representation based on image B-spline coefficients */
        Error = SamplesToCoefficients(buffer, (long)dim_y, (long)dim_x, (long)spline_degree);
        if (Error) {
            printf("Error in Bspline.dll : Change of basis failed\n");
            return(1);
        }

        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = (double)i - shifts[0];
            y0 = (double)j - shifts[1];

            if ((x0 <= -0.5) || (((double)dim_x - 0.5) <= x0)|| (y0 <= -0.5) || (((double)dim_y - 0.5) <= y0))
                b[p*dim_x*dim_y + i*dim_y + j] = 0.0;
            else
                b[p*dim_x*dim_y + i*dim_y + j] = (double)InterpolatedValue(buffer, (long)dim_y, (long)dim_x,y0, x0, (long)spline_degree);
        }}
    }
    delete [] buffer;
	return(0);
}

__declspec(dllexport) int Rotate(int dim, int dim_x, int dim_y, double* a, double* axis, double angle, int spline_degree, double* b)
{
    float *buffer = new float[dim_x*dim_y];

	double x0, y0;
	int Error;

    /* conversion from degrees to radians */
    //positive angles correspond to clockwise rotations
    angle = (PI/180.0) * angle;

    for (int p=0; p<dim; p++)
    {
        //puts image p in array buffer
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            buffer[i*dim_y + j] = a[p*dim_x*dim_y + i*dim_y + j];
        }}

        /* convert between a representation based on image samples */
        /* and a representation based on image B-spline coefficients */
        Error = SamplesToCoefficients(buffer, (long)dim_y, (long)dim_x, (long)spline_degree);
        if (Error) {
            printf("Error in Bspline.dll : Change of basis failed\n");
            return(1);
        }

        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = axis[0] + cos(angle)*((double)i - axis[0]) + sin(angle)*((double)j - axis[1]);
            y0 = axis[1] - sin(angle)*((double)i - axis[0]) + cos(angle)*((double)j - axis[1]);

            if ((x0 <= -0.5) || (((double)dim_x - 0.5) <= x0)|| (y0 <= -0.5) || (((double)dim_y - 0.5) <= y0))
                b[p*dim_x*dim_y + i*dim_y + j] = 0.0;
            else
                b[p*dim_x*dim_y + i*dim_y + j] = (double)InterpolatedValue(buffer, (long)dim_y, (long)dim_x,y0, x0, (long)spline_degree);
        }}
    }
    delete [] buffer;
	return(0);
}

__declspec(dllexport) int Scale(int dim, int dim_x, int dim_y, double* a, double* center, double factor, int spline_degree, double* b)
{
    float *buffer = new float[dim_x*dim_y];

	double x0, y0;
	int Error;

    for (int p=0; p<dim; p++)
    {
        //puts image p in array buffer
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            buffer[i*dim_y + j] = a[p*dim_x*dim_y + i*dim_y + j];
        }}

        /* convert between a representation based on image samples */
        /* and a representation based on image B-spline coefficients */
        Error = SamplesToCoefficients(buffer, (long)dim_y, (long)dim_x, (long)spline_degree);
        if (Error) {
            printf("Error in Bspline.dll : Change of basis failed\n");
            return(1);
        }

        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = ((double)i-center[0]) / factor + center[0];
            y0 = ((double)j-center[1]) / factor + center[1];

            if ((x0 <= -0.5) || (((double)dim_x - 0.5) <= x0)|| (y0 <= -0.5) || (((double)dim_y - 0.5) <= y0))
                b[p*dim_x*dim_y + i*dim_y + j] = 0.0;
            else
                b[p*dim_x*dim_y + i*dim_y + j] = (double)InterpolatedValue(buffer, (long)dim_y, (long)dim_x,y0, x0, (long)spline_degree);
        }}
    }
    delete [] buffer;
	return(0);
}
}

__declspec(dllexport) int Tilt_spectrum(int dim, int dim_x, int dim_y, double* a, double angle, double centre_spec, int spline_degree, double* b)
{
    float *buffer = new float[dim_x*dim_y];

	double x0, y0, slope;
	int Error;

    /* conversion from degrees to radians */
    //positive angles correspond to clockwise rotations
    slope = tan((PI/180.0) * angle);

    for (int p=0; p<dim; p++)
    {
        //puts image p in array buffer
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            buffer[i*dim_y + j] = a[p*dim_x*dim_y + i*dim_y + j];
        }}

        /* convert between a representation based on image samples */
        /* and a representation based on image B-spline coefficients */
        Error = SamplesToCoefficients(buffer, (long)dim_y, (long)dim_x, (long)spline_degree);
        if (Error) {
            printf("Error in Bspline.dll : Change of basis failed\n");
            return(1);
        }

        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = (double)i + slope*((double)j - centre_spec);
            y0 = (double)j;

            if ((x0 <= -0.5) || (((double)dim_x - 0.5) <= x0)|| (y0 <= -0.5) || (((double)dim_y - 0.5) <= y0))
                b[p*dim_x*dim_y + i*dim_y + j] = 0.0;
            else
                b[p*dim_x*dim_y + i*dim_y + j] = (double)InterpolatedValue(buffer, (long)dim_y, (long)dim_x,y0, x0, (long)spline_degree);
        }}
    }
    delete [] buffer;
	return(0);
}
